package com.atguigu.auth.service.impl;

import com.atguigu.auth.mapper.SysUserMapper;
import com.atguigu.auth.service.SysUserService;
import com.atguigu.common.utlis.JwtHelper;
import com.atguigu.constant.MessageConstant;
import com.atguigu.exception.BunnyException;
import com.atguigu.model.system.Login;
import com.atguigu.model.system.SysUser;
import com.atguigu.vo.system.LoginVo;
import com.atguigu.vo.system.SysUserQueryVo;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;
import org.springframework.util.StringUtils;

/**
 * <p>
 * 用户表 服务实现类
 * </p>
 *
 * @author bunny
 * @since 2024-04-22
 */
@Service
public class SysUserServiceImpl extends ServiceImpl<SysUserMapper, SysUser> implements SysUserService {

    /**
     * 用户条件分页查询
     *
     * @param page  当前页
     * @param limit 每页记录数
     * @param vo    查询条件
     * @return 分页结果
     */
    @Override
    public IPage<SysUser> getUserByPage(Long page, Long limit, SysUserQueryVo vo) {
        // 创建条件值
        String username = vo.getKeyword();
        String createTimeBegin = vo.getCreateTimeBegin();
        String createTimeEnd = vo.getCreateTimeEnd();

        // 创建分页对象
        Page<SysUser> pageParam = new Page<>(page, limit);

        // 分装查询对象
        LambdaQueryWrapper<SysUser> wrapper = new LambdaQueryWrapper<>();
        wrapper.like(!StringUtils.isEmpty(username), SysUser::getUsername, username);
        // 大于
        wrapper.ge(!StringUtils.isEmpty(createTimeBegin), SysUser::getCreateTime, createTimeBegin);
        // 小于
        wrapper.le(!StringUtils.isEmpty(createTimeEnd), SysUser::getCreateTime, createTimeEnd);

        return page(pageParam, wrapper);
    }

    /**
     * 更新状态
     *
     * @param id     用户id
     * @param status 修改的状态
     */
    @Override
    public void updateStatus(Long id, Long status) {
        SysUser sysUser = getById(id);
        if (status == 1) sysUser.setStatus(1);
        else sysUser.setStatus(0);

        // 更新用户状态
        updateById(sysUser);
    }

    /**
     * 登录
     *
     * @param vo 登录条件
     * @return 返回token
     */
    @Override
    public Login login(LoginVo vo) {
        String username = vo.getUsername();
        String password = vo.getPassword();
        // 查询用户信息
        LambdaQueryWrapper<SysUser> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(SysUser::getUsername, username);
        SysUser sysUser = getOne(wrapper);

        // 用户是否存在
        if (sysUser == null) {
            throw new BunnyException(MessageConstant.ACCOUNT_NOT_FOUND);
        }

        // 判断是否被禁用
        if (sysUser.getStatus() == 0) {
            throw new BunnyException(MessageConstant.ACCOUNT_LOCKED);
        }

        // 判断密码
        String md5DigestAsHexPassword = DigestUtils.md5DigestAsHex(password.getBytes());
        if (!md5DigestAsHexPassword.equals(password)) {
            throw new BunnyException(MessageConstant.PASSWORD_ERROR);
        }
        // 添加token
        String token = JwtHelper.createToken(sysUser.getId(), sysUser.getUsername());
        return Login.builder().token(token).build();
    }
}
